@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 import warnings
 from contextlib import asynccontextmanager
-from typing import Union, Iterable
+from typing import Union, Any, Tuple
 
 from google.protobuf.wrappers_pb2 import BytesValue, StringValue
 from patchwork.core import Task
@@ -27,14 +27,14 @@ class TaskCatcher:
         self._tasks.append(task)
 
     @property
-    def tasks(self) -> Iterable[Task]:
+    def tasks(self) -> Tuple[Task]:
         """
         Returns a list of caught tasks
         :return:
         """
         return tuple(self._tasks)
 
-    def _compare_tasks(self, task: Task, expected_task):
+    def compare_tasks(self, task: Task, expected_task: Union[Task, dict]):
 
         hasser = hasattr
         getter = getattr
@@ -90,6 +90,8 @@ class TaskCatcher:
                 if getattr(task.meta, meta) != getter(expected_meta, meta):
                     delta.add(f'meta.{meta}')
 
+        return delta
+
     def assert_processed(self, expected_task: Union[Task, dict], only: bool = False, count: int = None):
         """
         Check if expected task as been executed. Only set attributes (items for dict) of expected task
@@ -109,7 +111,7 @@ class TaskCatcher:
         matching = []
 
         for task in self._tasks:
-            delta = self._compare_tasks(task, expected_task)
+            delta = self.compare_tasks(task, expected_task)
             if not delta:
                 matching.append(task)
 
@@ -186,6 +188,10 @@ class TestWorker:
         self.worker.executor.on_event('task')(self._on_task)
         self._on_task_callback = None
 
+    @property
+    def broker(self) -> AsyncLocalBroker:
+        return self._get_publisher().broker
+
     async def __aenter__(self):
         await self.worker.run()
         return self
@@ -210,6 +216,63 @@ class TestWorker:
 
         return lag
 
+    def _get_publisher(self) -> AsyncLocalPublisher:
+        return self.worker.get_publisher()
+
+    async def send(self, payload: Any, *, timeout: float = None, cause: Task = None, task_type: str = None, **meta):
+        return await self._get_publisher().send(payload, timeout=timeout, cause=cause, task_type=task_type, **meta)
+
+    async def send_task(self, task: Task, *, timeout: float = None):
+        return await self._get_publisher().send_task(task, timeout=timeout)
+
+    async def wait_once(self) -> bool:
+        """
+        Wait until one task has been processed.
+        :returns: True if there is no lag on broker or False if there is a lag (there is more tasks to process)
+        """
+
+        if not self.worker.executor.busy.value:
+            if self._get_subscriber_lag() == 0:
+                # worker is not busy and there is nothing on queues to process
+                return True
+
+            # await until executor becomes busy
+            await self.worker.executor.busy.wait_for(True)
+
+        # await until executor completes task processing
+        await self.worker.executor.busy.wait_for(False)
+
+        return self._get_subscriber_lag() == 0
+
+    async def wait_until_completed(self):
+        """
+        Wait until all tasks pending at local broker will be completed. If one of processing tasks schedule
+        more tasks, awaits also on them.
+        :return:
+        """
+        while True:
+            if await self.wait_once():
+                break
+
+    async def wait_until(self, task: Union[Task, dict]):
+        """
+        Executes tasks until expected one is processed. If not reached and no messages left on the queue,
+        raises AssertionError.
+        :param task:
+        :return:
+        """
+        tc = TaskCatcher()
+        self._on_task_callback = tc.feed
+        more_todo = True
+
+        while more_todo:
+            more_todo = await self.wait_once()
+
+            if not tc.compare_tasks(tc.tasks[-1], task):
+                return
+
+        raise AssertionError(f"no messages left and expected task has been not processed")
+
     @asynccontextmanager
     async def catch_tasks(self) -> TaskCatcher:
         tc = TaskCatcher()
@@ -217,15 +280,5 @@ class TestWorker:
 
         yield tc
 
-        while True:
-
-            if not self.worker.executor.busy.value:
-                if self._get_subscriber_lag() == 0:
-                    # worker is not busy and there is nothing on queues to process
-                    return
-
-                # await until executor becomes busy
-                await self.worker.executor.busy.wait_for(True)
-
-            # await until executor completes task processing
-            await self.worker.executor.busy.wait_for(False)
+        await self.wait_until_completed()
+        self._on_task_callback = None
